import pathlib as pl
import scipy.signal as sig
import scipy as sci
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import soundfile as sf
import os
from pyts.decomposition import SingularSpectrumAnalysis
from numpy.fft import rfft,rfftfreq
from tqdm import tqdm
import pandas as pd
import matplotlib


def downsample_and_save_50hz():
    decimatefactors=[2,3,5,6,7]
    days=[f"{i:02}" for i in range(1,32)]
    for day in tqdm(days):
        path = pl.Path("RawData/march/").joinpath(day)        
        for file in pl.Path("RawData/march/").joinpath(day).glob("**/*"):
            #print(file.name)
            data, _ = sf.read(file)
            y1, y2= data[:44100*60, 0], data[:44100*60, 1]
            for i in decimatefactors:
                y1 = sci.decimate(y1, i)
                y2 = sci.decimate(y2, i)
            sop_diff = y2 - y1
            sop_sum = y1 + y2
            sop=np.row_stack([sop_diff,sop_sum])
            savepath=pl.Path("35hz_data/march/").joinpath(day)
            if not savepath.exists(): 
                savepath.mkdir()
            savepath = savepath.joinpath(file.name[:-5]+".npy")
            np.save(savepath,sop)


def load_week(month,dayrange1,dayrange2):
    sumlist=[]
    difflist=[]
    monthstr="202212" if month=="des" else "202303"
    for file in pl.Path(f"210hz_data/{month}/").glob(f"**/output-{monthstr}[{dayrange1}][{dayrange2}]*"):
        #print(file.name)
        data = np.load(file)
        difflist.append(data[0,10:-10])
        sumlist.append(data[1,10:-10])

    difflist=np.row_stack(difflist)
    sumlist=np.row_stack(sumlist)
    return difflist,sumlist



def load_35hz(month,dayrange1,dayrange2):
    sumlist=[]
    difflist=[]
    monthstr="202212" if month=="des" else "202303"
    for file in pl.Path(f"35hz_data/{month}/").glob(f"**/output-{monthstr}[{dayrange1}][{dayrange2}]*"):
        #print(file.name)
        data = np.load(file)
        difflist.append(data[0,10:-10])
        sumlist.append(data[1,10:-10])

    difflist=np.row_stack(difflist)
    sumlist=np.row_stack(sumlist)
    return difflist,sumlist


def load_35hz2(month,dayrange1,dayrange2):
    sumlist=[]
    difflist=[]
    monthstr="202212" if month=="des" else "202303"
    for file in pl.Path(f"35hz_data/{month}/").glob(f"**/output-{monthstr}[{dayrange1}][{dayrange2}]*"):
        #print(file.name)
        data = np.load(file)
        difflist.append(data[0,:])
        sumlist.append(data[1,:])

    difflist=np.row_stack(difflist)
    sumlist=np.row_stack(sumlist)
    return difflist,sumlist


def plot_contour(data,ndays):
    diff_fft = rfft(data, axis=1)
    times=np.arange(np.datetime64("2022-12-01T00:00"), np.datetime64("2022-12-01T00:00")+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(data.shape[0]-1),"ms"))

    freqs = rfftfreq(data.shape[1], 1/210)
    print(diff_fft.shape)
    print(freqs.shape)
    for i in diff_fft:
        break
        plt.plot(freqs,np.abs(i))
        plt.xlim(45,55)
        plt.show()
    size=45
    lvls=np.arange(0, 10000, 25)
    plt.figure(figsize=(16/25*size,9/25*size))
    plt.contour(times, freqs,(np.abs(diff_fft.T)),levels=lvls,cmap="jet")
    plt.colorbar()
    plt.ylim(49.75, 50.25)
    plt.savefig(f"plots/{ndays}_spectrogram_0.5hz.png")
    plt.savefig(f"plots/{ndays}_spectrogram_0.5hz.pdf")

    plt.figure(figsize=(16/25*size,9/25*size))
    plt.contour(times,freqs,(np.abs(diff_fft.T)),levels=lvls,cmap="jet")
    plt.colorbar()
    plt.ylim(49.5,50.5)
    plt.savefig(f"plots/{ndays}_spectrogram_1hz.png")
    plt.savefig(f"plots/{ndays}_spectrogram_1hz.pdf")
    plt.figure(figsize=(16/25*size,9/25*size))
    plt.contour(times,freqs,(np.abs(diff_fft.T)),levels=lvls,cmap="jet")
    plt.colorbar()
    plt.ylim(49.9,50.1)

    plt.savefig(f"plots/{ndays}_spectrogram_0.1hz.png")
    plt.savefig(f"plots/{ndays}_spectrogram_0.1hz.pdf")
    plt.show(block=True)

    




def plot_sidelobes(data, ndays, lower_freq=50.1, upper_freq=70,starttime="2022-12-01T00:00",normalized= True):
    fft = rfft(data,axis=1)
    freqs = rfftfreq(data.shape[1],1/210)
    index_array_01hz = (49.9 < freqs) & (freqs < 50.1) 
    #lower_sidelobe_index_array = ((lower_sidelobe_freq < freqs) & (freqs < lower_normal_freq)) 
    upper_sidelobe_index_array = ((lower_freq < freqs ) & (freqs<upper_freq))
    

    times = np.arange(np.datetime64(starttime), np.datetime64(starttime)+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(fft.shape[0]-1),"ms"))
    avg_50hz = np.average(np.abs(fft[:, index_array_01hz]), axis=1)

    #lowlvls = np.linspace(np.min(np.log(np.abs(fft[:,lower_sidelobe_index_array]).T/avg_50hz)),np.max(np.log(np.abs(fft[:,lower_sidelobe_index_array]).T/avg_50hz)),100)
    highlvls = np.linspace(np.min(np.log(np.abs(fft[:,upper_sidelobe_index_array]).T/avg_50hz)),np.max(np.log(np.abs(fft[:,upper_sidelobe_index_array]).T/avg_50hz)),100)
    #log_low_lobe=np.log(np.abs(fft[:,lower_sidelobe_index_array]).T/avg_50hz)
    log_upp_lobe= np.log(np.abs(fft[:,upper_sidelobe_index_array]).T/avg_50hz)
    
    #nnlowlvls = np.linspace(np.min(np.log(np.abs(fft[:, lower_sidelobe_index_array]))), np.max(np.log(np.abs(fft[:, lower_sidelobe_index_array]))),100)
    nnhighlvls = np.linspace(np.min(np.log(np.abs(fft[:, upper_sidelobe_index_array]))), np.max(np.log(np.abs(fft[:, upper_sidelobe_index_array]))),1000)

    #nn_log_low_lobe=np.log(np.abs(fft[:,lower_sidelobe_index_array])).T
    nn_log_upp_lobe=np.log(np.abs(fft[:,upper_sidelobe_index_array])).T


    """plt.figure(figsize=(16,10))
    plt.plot(times,avg_50hz)
    plt.title("Avg 50Hz for march")
    plt.xlabel("Date")
    plt.ylabel("amplitude")
    plt.show()

    print(fft.shape)
    print(times.shape)
    print(freqs.shape)
    print(log_low_lobe.shape)
    print(nn_log_low_lobe.shape)
    plt.figure(figsize=(16,10))
    plt.contour(times,
                freqs[index_array_2hz_lower],
                nn_log_low_lobe, levels=nnlowlvls, cmap="jet")
    plt.title("Spectrogram of the lower sidelobe")
    plt.xlabel("date")
    plt.ylabel("frequency")
    plt.colorbar()
    plt.savefig(f"lower sidelobe_startdate={starttime}_ndays={ndays}.png")
    #plt.savefig(f"lower sidelobe_startdate={starttime}_ndays={ndays}.pdf")
    
    plt.show()"""

    plt.figure(figsize=(16,9))
    plt.contour(times,
                freqs[upper_sidelobe_index_array],
               log_upp_lobe,levels=highlvls,cmap="jet")
    plt.title("Spectrogram")
    plt.xlabel("Date")
    plt.ylabel("Frequency")
    plt.colorbar()
    plt.show()

    quit()
    plt.figure(figsize=(16,9))
    plt.contour(times,
                freqs[lower_sidelobe_index_array],
               log_low_lobe,levels=lowlvls,cmap="jet")
    plt.title("Spectrogram of the upper sidelobe")
    plt.xlabel("date")
    plt.ylabel("frequency")
    plt.colorbar()
    #plt.savefig(f"upper sidelobe_startdate={starttime}_ndays={ndays}.png")
    #plt.savefig(f"upper sidelobe_startdate={starttime}_ndays={ndays}.pdf")
    
    plt.show()



def plot_temp_and_ranged_spectrogram(data,ndays,starttime,freq_range,tempplot=0, normalized=False):
    fft = rfft(data,axis=1)
    freqs = rfftfreq(data.shape[1],1/210)
    index_array = (freq_range[0] < freqs) & (freqs < freq_range[1]) 
    print(fft.shape)
    
    #fft=fft/np.average(fft[:,carrierband])
    
    
    times = np.arange(np.datetime64(starttime), np.datetime64(starttime)+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(fft.shape[0]-1),"ms"))
    if normalized:
        carrierband=(49.9< freqs) & (freqs < 50.1)
        loglevels = np.log(np.abs(fft[:,index_array]).T/np.average(fft[:,carrierband]))
        low_lvls = np.linspace(np.min(np.log(np.abs(fft[:,index_array]).T/np.average(fft[:,carrierband]))),np.max(np.log(np.abs(fft[:,index_array]).T/np.average(fft[:,carrierband]))),100)
        print(low_lvls)
    else:
        loglevels = np.log(np.abs(fft[:,index_array])).T
    lvls = np.linspace(np.min(loglevels),np.max(loglevels),100)
    
    print(lvls)
    print(loglevels.shape)
    
    
    if tempplot:
        df=pd.read_csv("weatherdata_march.csv",sep=";",skipfooter=1)
        print(df["Lufttemperatur"])
        temps=df["Lufttemperatur"]
        plt.plot(np.linspace(1,31,len(temps)),temps)
        plt.title("Temperature for march")
        plt.xlabel("Date")
        plt.ylabel("Temperature")
        plt.show()
    
    upperbound_df = pd.read_csv("upperbound_vals.txt",names=("values","times"))
    upperbound_times=pd.to_datetime(upperbound_df["times"])
    upperbound=upperbound_df["values"]
    lowerbound_df = pd.read_csv("lowerbound_vals.txt",names=("values","times"))
    lowerbound_times=pd.to_datetime(lowerbound_df["times"])

    #print(lowerbound_times.dtype)
    #lowerbound_times=np.datetime64(lowerbound_times)

    lowerbound=lowerbound_df["values"]

    #fig, ax = plt.subplots()
    #fig.set_size_inches(8*1.5,5*1.5)

    
    plt.figure(figsize=(16,8))
    plt.contour(times,freqs[index_array],loglevels,levels=lvls,cmap="jet")
    plt.colorbar()
    #plt.plot(upperbound_times, upperbound,"blue")
    #plt.plot(lowerbound_times, lowerbound,"blue")

    plt.title(f"Spectrogram of frequency range {freq_range}")
    plt.xlabel("Date")
    plt.ylabel("Frequency")

    #plt.savefig(f"spectrogram {freq_range}_start{starttime[:4]}_normalized_{normalized}.png")
    plt.show()
    
    

def track_bands(data,ndays,starttime,freq_range):
    fft = rfft(data,axis=1)
    freqs = rfftfreq(data.shape[1],1/210)
    index_array = (freq_range[0] < freqs) & (freqs < freq_range[1]) 
    loglevels = np.log(np.abs(fft[:,index_array])**2).T


    upperbound_df = pd.read_csv("upperbound_vals.txt",names=("values","times"))
    upperbound=upperbound_df["values"]
    ub_ns=np.array([int(time.value) for time in pd.to_datetime(upperbound_df["times"])])
    


    lowerbound_df = pd.read_csv("lowerbound_vals.txt",names=("values","times"))
    lowerbound=lowerbound_df["values"]
    lb_ns=np.array([int(time.value) for time in pd.to_datetime(lowerbound_df["times"])])
    #print(fft.shape[0])

    times = np.arange(np.datetime64(starttime), np.datetime64(starttime) + np.timedelta64(ndays,"D"), np.timedelta64((ndays) * 86400 * 1000 // (fft.shape[0]-1),"ms"))
    lb_interp = np.interp(np.linspace(lb_ns[0],lb_ns[-1], len(times)), lb_ns, lowerbound)
    ub_interp = np.interp(np.linspace(ub_ns[0],ub_ns[-1], len(times)), ub_ns, upperbound)
    peaks = []
    peakvals=[]
    for ind,bounds in enumerate(zip(lb_interp,ub_interp)):
        vals = np.where((bounds[0] < freqs) & (bounds[1] > freqs),(np.abs(fft[ind,:])),0)
        peakvals.append(np.max(vals))
        peaks.append(freqs[np.argmax(vals)])
    plt.plot(rfftfreq(len(peakvals),60),np.abs(rfft(peakvals-np.mean(peakvals))))
    plt.xlabel("frequency")
    plt.ylabel("a.u.")
    plt.show()
    plt.plot(times,peakvals)

    plt.title("Peak values")
    plt.xlabel("Time")
    plt.ylabel("PDS [dB/Hz]")
    plt.show()
    plt.plot(times,peaks)
    plt.plot(pd.to_datetime(upperbound_df["times"]), upperbound,"blue")
    plt.plot(pd.to_datetime(lowerbound_df["times"]), lowerbound,"blue")
    plt.show()
    #print(peaks)
    L=100
    groups=[[0],[1],[2],[3],[4],[5],[6,7,8,9,10,11],[12,13,14,15,16,17,18,19,20]]
    SSA = SingularSpectrumAnalysis(window_size=L, groups=groups)
    print()
    
    ssa_signal=SSA.fit_transform(np.array(peaks).reshape(1,-1))
    for ind,signal in enumerate(ssa_signal):
        plt.plot(times,signal)
        if ind>=1:
            break
        plt.xlabel("Time")
        plt.ylabel("signal")
        plt.title(f" SSA component number {ind}")
        plt.legend()
        plt.show()

    for ind,cumsig in enumerate(np.cumsum(ssa_signal,axis=0)):
        plt.plot(times,cumsig,label=f"cumulative signal {ind}")
        if ind>=1:
            break
    plt.xlabel("Time")
    plt.ylabel("signal")
    plt.title("Cumulative sum of the first 2 SSA components")
    plt.legend()
    plt.show()
    #plt.plot(times,np.sum(ssa_signal,axis=0),label="ssa_total")


    #print(len(peaks))
    #peaks_moving_avg=np.convolve(peaks,[1/300]*300,mode="same")
    #plt.plot(times,peaks_moving_avg)
    cum_ssa_sig = np.cumsum(ssa_signal,axis=0)
    trend_crossings=np.where(np.diff(np.sign(cum_ssa_sig[6]-cum_ssa_sig[0])))[0]
    print(trend_crossings)

    """plt.plot(times,peaks,label="raw signal")
    plt.xlabel("Time")
    plt.ylabel("Signal strength")
    plt.title("Tracked peak")
    plt.legend()
    plt.show()

    plt.plot(times,np.sign(cum_ssa_sig[6]-cum_ssa_sig[0]))
    plt.show()
    """
    fft_peaks = rfft(peaks)
    peak_freqs = rfftfreq(len(peaks),60)
    """
    plt.plot(peak_freqs,np.log(np.abs(rfft(np.sign(cum_ssa_sig[6]-cum_ssa_sig[0])))),label="sign fft")
    plt.plot(peak_freqs,np.log(np.abs(fft_peaks)),label="signal fft")
    plt.title("Tracked peak PDS")
    plt.xlabel("Frequency")
    plt.ylabel("Power density")
    plt.legend()
    plt.show()    
    """
    spec,freqsuency,t,im=plt.specgram(np.sign(cum_ssa_sig[6]-cum_ssa_sig[0]),Fs=1/60,cmap="jet")
    
    plt.title("spectrogram of sign signal")
    plt.ylabel("Frequency")
    plt.xlabel("time")
    #plt.xticks(t,times[:39680:128])
    plt.colorbar()
    plt.show()
    print(np.insert(t,0,0))
    plt.specgram(ssa_signal[1],Fs=1/60,cmap="jet")
    plt.title("spectrogram of second SSA component")
    plt.ylabel("Frequency")
    plt.xlabel("time")
    #plt.xticks(t[:-1:2],times[:39680+128*2:128*2],rotation=45)
    plt.colorbar()
    plt.show()
    ssa_summed_signal = np.sum(ssa_signal[1:6],axis=0)
    plt.specgram(ssa_summed_signal,Fs=1/60,cmap="jet")
    plt.title("spectrogram of sum of components 2-4")
    plt.ylabel("Frequency")
    plt.xlabel("time")
    #plt.xticks(t[::30],times[:39680:128*30],rotation=45)
    plt.colorbar()
    plt.show()

    quit()
    fft = rfft(data,axis=1)
    lvls=np.linspace(np.min(loglevels),np.max(loglevels),100)
    fig, ax = plt.subplots()
    ax.contour(times,freqs[index_array],loglevels,levels=lvls,cmap="jet")
    ax.plot(times,peaks)
    #plt.colorbar(ax=ax)
    #ax.plot(upperbound_times, upperbound,"blue")
    #ax.plot(lowerbound_times, lowerbound,"blue")

    ax.set_title(f"Spectrogram of frequency range")
    ax.set_xlabel("Date")
    ax.set_ylabel("Frequency")
    plt.show()


def time_domain_analysis(data,sum=False):
    sos = sig.butter(5,(68,71),"bandpass",output="sos",fs=210)
    sos2 = sig.butter(5,(68,71),"bandstop",output="sos",fs=210)
    mincount=30
    #plt.plot(np.linspace(0,60*100,len(data[:100,:].flatten())),data[:100,:].flatten(),label="original signal")
    #plt.plot(np.linspace(0,60*mincount,len(data[:mincount,:].flatten())),sig.sosfilt(sos2,data[:mincount,:].flatten()),label="bandstop 68-71hz")
    plt.plot(np.linspace(0,60*mincount,len(data[:mincount,:].flatten())),sig.sosfilt(sos,data[:mincount,:].flatten()),label="bandpass 68-71hz")
    plt.legend()
    plt.xlabel("time [sec]")
    plt.ylabel("signal strength")
    if sum:
        plt.title("Bandpassed time domain of Ax+Ay")
    else:
        plt.title("Bandpassed time domain of Ax-Ay")
    plt.plot(np.arange(0,60*mincount,60), np.zeros_like(np.arange(0,60*mincount,60)),"rx",markersize=3,label="minute markers") 
    plt.show()


def spec_only_start_of_segments(data,nsecs,starttime="2023-03-01T00:00",ndays=31):
    sos = sig.butter(20,(68,71),"bandpass",output="sos",fs=210)
    cutdata = sig.sosfilt(sos,data[:,:210*nsecs],axis=1)
    cutfft = np.log(np.abs(rfft(cutdata,axis=1)))

    plt.plot(np.linspace(0,len(data[:,210*nsecs].T.flatten())/210,len(data[:,210*nsecs].flatten())),sig.sosfilt(sos,data[:,210*nsecs].flatten()))
    plt.show()
    plt.plot(rfftfreq(len(sig.sosfilt(sos,data[:,210*nsecs].flatten())),1/210), np.log(np.abs(rfft(sig.sosfilt(sos,data[:,210*nsecs].flatten())))))  
    plt.show()             
    print(cutfft.shape)
    print(cutdata.shape)

    cutfreqs = rfftfreq(cutdata.shape[1],1/210)
    print(cutfreqs.shape)
    freq_range=(65,75)
    index_array = (freq_range[0] < cutfreqs) & (cutfreqs < freq_range[1]) 
    times = np.arange(np.datetime64(starttime), np.datetime64(starttime) + np.timedelta64(ndays,"D"), np.timedelta64((ndays) * 86400 * 1000 // (cutfft.shape[0]-1),"ms"))
    lvls=np.linspace(np.min(cutfft[:,index_array].T),np.max(cutfft[:,index_array].T),100)

    plt.contour(times,cutfreqs[index_array],cutfft[:,index_array].T,levels=lvls,cmap="jet")
    plt.colorbar()
    plt.show()


def bandpass_energy(data,freq_range,starttime="2023-03-01T00:00",ndays=31):
    fft = rfft(data,axis=1)
    freqs = rfftfreq(data.shape[1],1/210)
    print(freqs[1])

    index_array = (freq_range[0] < freqs) & (freqs < freq_range[1]) 
    index_array50 = (49.9 < freqs) & (freqs < 50.1)
    index_array_low = (freq_range[0]-50 < freqs) & (freqs < freq_range[1]-50)
    """
    sos = sig.butter(10,freq_range,"bandpass",output="sos",fs=210)
    filtdata=sig.sosfilt(sos,data,axis=1)
    times = np.arange(np.datetime64(starttime), np.datetime64(starttime)+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(data.shape[0]-1),"ms"))
    plt.plot(times,np.mean(filtdata.T,axis=0))
    plt.show()
    """

    avg = np.average(np.abs(fft[:, index_array]), axis=1)
    avg50 =np.average(np.abs(fft[:, index_array50]), axis=1)
    avg_low  = np.average(np.abs(fft[:, index_array_low]), axis=1)
    times = np.arange(np.datetime64(starttime), np.datetime64(starttime)+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(fft.shape[0]-1),"ms"))
    pdtimes= pd.DatetimeIndex(times)
    busdays= pdtimes.day_of_week<=4
    #plt.plot(pdtimes,np.array(busdays,dtype="int64")*(1.11),label="bussiness days")
    #plt.plot(pdtimes,avg50,label="50hz signal",alpha=0.4)
    plt.plot(pdtimes,avg_low,label=f"{freq_range[0]-50:.1f}-{freq_range[1]-50}Hz signal")

    rushhours = [True if t in range(6,9) or t in range(15,18) else False for t in pdtimes.hour]
    nighttime= [True if t in range(23,25) or t in range(0,6) else False for t in pdtimes.hour]

    busdayrush=busdays*rushhours

    print("Percent of time spent as night" , np.mean(nighttime))
    print("percent of time spent as business day rushhour:", np.mean(busdayrush))

    print(freq_range[0]-50,freq_range[1]-50)
    print("Businessday rush hour avg energy: ", np.sum(avg_low[busdayrush])/np.sum(avg_low))
    print("Not businessday rush hour avg energy: ", np.sum(avg_low[~busdayrush])/np.sum(avg_low))

    print("Night time avg energy: ", np.sum(avg_low[nighttime])/np.sum(avg_low))
    print("Day time avg energy: ", np.sum(avg_low[~np.array(nighttime)])/np.sum(avg_low))
    #plt.plot(pdtimes,np.array(rushhours,dtype="int64")*1.1,label="rushhours")
    plt.plot(pdtimes,np.array(nighttime,dtype="int64")*1.1*np.max([max(avg_low),max(avg)]),label="Night time")
    #plt.plot(pdtimes,np.array(busdayrush,dtype="int64")*1.1*np.max([max(avg_low),max(avg)]),label="businessday rush hours")
    #plt.plot(pdtimes,avg50/np.max(avg50),label="normalized average 50hz energy",alpha=0.3)
    plt.plot(pdtimes,avg,label=f"Normalized average signal energy in {freq_range}Hz",alpha=0.7)
    
    plt.legend()
    plt.xlabel("Time")
    plt.ylabel("value")
    plt.title(f"Frequency band energy, Night time marked")
    #plt.show()



def peak_track_corr_temp(data, tempfile, freq_range, starttime="2023-03-01T00:00", ndays=31):
    fft = np.abs(rfft(data,axis=1))
    freqs = rfftfreq(data.shape[1],1/210)
    index_array = (freq_range[0] < freqs) & (freqs < freq_range[1]) 
    times = np.arange(np.datetime64(starttime), np.datetime64(starttime)+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(fft.shape[0]-1),"ms"))
    pdtimes= pd.DatetimeIndex(times)

    weatherdata=pd.read_csv(tempfile,sep=";",skipfooter=1)
    # np.interp(np.linspace(lb_ns[0],lb_ns[-1], len(times)), lb_ns, lowerbound)
    avg=np.average(fft[:,index_array],axis=1)
    peaks, _ = sig.find_peaks(avg,distance=500,prominence=100)
    newpoints=np.linspace(peaks[0],peaks[-1],len(times))
    extraweather=np.interp(newpoints,np.linspace(0,31*60*24,len(weatherdata["Lufttemperatur"])),weatherdata["Lufttemperatur"])
    
    #extraweather=sci.interpolate.interp1d(np.linspace(1,31,len(weatherdata["Lufttemperatur"])),weatherdata["Lufttemperatur"])(np.linspace(0,31,len(times)))
    
    interp_peaks = np.interp(newpoints,peaks,avg[peaks])
    win = 60
    win2 = 120
    moving_avg = np.convolve(avg,win*[1/win], "same")
    inter_mvg_avg= np.interp(newpoints,np.linspace(1,31*24*60,len(moving_avg)),moving_avg)
    moving_avg2 = np.convolve(avg,win2*[1/win2], "same")
    inter_mvg_avg2= np.interp(newpoints,np.linspace(1,31*24*60,len(moving_avg2)),moving_avg2)
    plt.plot(times,avg,label="avg")
    plt.plot(times[peaks],avg[peaks],"rx",label="peaks")
    plt.plot(times,extraweather,label="weather")
    plt.plot(times, interp_peaks,label="interpolated_peaks")
    plt.plot(times,inter_mvg_avg,label=f"interpolated moving avgerage window {win}")
    plt.plot(times,inter_mvg_avg2,label=f"interpolated moving avgerage window {win2}")
    plt.legend()
    plt.show()


    plt.plot(np.correlate(extraweather-np.mean(extraweather),inter_mvg_avg-np.mean(inter_mvg_avg),"full"),label="corr interpolated moving avg")
    plt.plot(np.correlate(extraweather-np.mean(extraweather),avg-np.mean(avg),"full"),label="corr avg")
    plt.plot(np.correlate(extraweather-np.mean(extraweather),interp_peaks-np.mean(interp_peaks),"full"),label="corr interp_peaks")
    plt.legend()
    
    
    print(
        np.correlate(extraweather-np.mean(extraweather),inter_mvg_avg-np.mean(inter_mvg_avg)),
        np.correlate(extraweather-np.mean(extraweather),avg-np.mean(avg)),
        np.correlate(extraweather-np.mean(extraweather),interp_peaks-np.mean(interp_peaks))
        )
    plt.show()
    input("plot thickened")
    

def plot50hz_and_power_data(data,starttime="2023-03-01T00:00",endtime="2023-03-31 23:00:00",ndays=31,sumvar=False):
    fft = np.abs(rfft(data,axis=1))
    freqs = rfftfreq(data.shape[1],1/210)
    index_array = (49.9 < freqs) & (freqs < 50.1) 
    avg=np.average(fft[:,index_array],axis=1)
    times = np.arange(np.datetime64(starttime), np.datetime64(starttime)+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(fft.shape[0]-1),"ms"))
    pdtimes= pd.DatetimeIndex(times)
    importdata= pd.read_csv("ImportExport-2023.csv") if starttime=="2023-03-01T00:00" else pd.read_csv("ImportExport-2022.csv")
    try:
        importdata["Time(Local)"]=pd.date_range(start=pd.Timestamp(importdata.loc[0,"Time(Local)"]),end=pd.Timestamp(importdata.loc[importdata.shape[0]-1,"Time(Local)"]),freq="H")
    except:
        importdata["Time(Local)"]=pd.date_range(start=pd.Timestamp(importdata.loc[0,"Time(Local)"]),end=pd.Timestamp(importdata.loc[importdata.shape[0]-2,"Time(Local)"]),freq="H")
    destimes=importdata["Time(Local)"].isin(pd.date_range(start=pd.Timestamp(starttime),end=pd.Timestamp(endtime),freq="H")).to_numpy()
    importdata["SumImportExport"] = importdata["Import"] + importdata["Export"]
    print(destimes.shape)
    print(pdtimes.shape)
    df=pd.Series(avg,index=pdtimes)

    #print(df.resample("H").sum().head())
    print(df.resample("H").mean())

    #interpolate_Impexp= np.interp(np.linspace(0,31,len(times)),np.linspace(0,31,len(importdata["SumImportExport"])),importdata["SumImportExport"].to_numpy())
    #interpolate_Imp= np.interp(np.linspace(0,31,len(times)),np.linspace(0,31,len(importdata["Import"])),importdata["Import"])
    #interpolate_Exp= np.interp(np.linspace(0,31,len(times)),np.linspace(0,31,len(importdata["Export"])),importdata["Export"])
    win=15
    powerdata = pd.read_csv("ProductionConsumption2023.csv") if starttime=="2023-03-01T00:00" else pd.read_csv("ProductionConsumption2022.csv")

    #interp=sci.interpolate.interp1d()
    print("Correlation coefficients import vs 50hz",np.corrcoef(df.resample("H").mean(),importdata.loc[destimes,"Import"]))
    print("Correlation coefficients export vs 50hz",np.corrcoef(df.resample("H").mean(),importdata.loc[destimes,"Export"]))
    print("Correlation coefficients sum import export vs 50hz", np.corrcoef(importdata.loc[destimes,"SumImportExport"],df.resample("H").mean()))
    font= {'weight' : 'normal','size'   : 16}
    plt.rc("font",**font)
    mediumsize=16
    plt.rc("xtick",labelsize=mediumsize-2)
    plt.rc("ytick",labelsize=mediumsize-2)
    plt.rc("axes",labelsize=mediumsize)
    plt.figure(figsize=(16,9))
    #plt.plot(times,interpolate_Impexp)
    #plt.plot(times,np.convolve(avg,win*[1/win],"same"))

    print("Correlation coefficients; Consumption vs 50hz",np.corrcoef(df.resample("H").mean(),powerdata.loc[destimes,"Consumption"]))
    print("Correlation coefficients: Production vs 50hz",np.corrcoef(df.resample("H").mean(),powerdata.loc[destimes,"Production"]))
    print("Correlation coefficients: surplus vs 50hz",np.corrcoef(df.resample("H").mean(),powerdata.loc[destimes,"Production"]-powerdata.loc[destimes,"Consumption"]))
    print("Correlation coefficients: Deficit  vs 50hz",np.corrcoef(df.resample("H").mean(),powerdata.loc[destimes,"Consumption"]-powerdata.loc[destimes,"Production"]))
    #print("Correlation coefficients sum import export vs 50hz", np.corrcoef(importdata.loc[destimes,"SumImportExport"],df.resample("H").mean()))
    fig,ax=plt.subplots(figsize=(16,9))
    ax.plot(powerdata.loc[destimes,'Time(Local)'],df.resample("H").mean(),label="50hz signal")
    #plt.plot(powerdata.loc[destimes,'Time(Local)'],,"tab:green",label="National Power consumption")
    ax.plot(powerdata.loc[destimes,'Time(Local)'],powerdata.loc[destimes,"Production"]-powerdata.loc[destimes,"Consumption"],"tab:green", label="National Power deficit/surplus")
    #plt.plot(importdata.loc[destimes,'Time(Local)'],importdata.loc[destimes,"SumImportExport"], label="Import + Export")
    #plt.plot(importdata.loc[destimes,'Time(Local)'],importdata.loc[destimes,"Export"],"tab:green", label="National Power Export")
    #plt.plot(importdata.loc[destimes,'Time(Local)'],importdata.loc[destimes,"Import"],"tab:red", label="National Power Import")
    ax.set_xlabel("Time")
    ax.set_ylabel("Signal energy // Power [au/Hz // MWh]")
    ax.set_title("50Hz signal vs National power deficit/surplus")
    fig.autofmt_xdate() 
    ax.xaxis.set_major_locator(matplotlib.ticker.MaxNLocator(nbins=25))
    fig.legend(loc="upper left")
    #plt.show()
    


    fig,ax1 = plt.subplots(figsize=(16,9))
    ax2 = ax1.twinx()
    
    ax1.plot(powerdata.loc[destimes,'Time(Local)'],df.resample("H").mean(),label="50hz signal")
    #ax2.plot(powerdata.loc[destimes,'Time(Local)'],powerdata.loc[destimes,"Consumption"],"tab:green",label="National Power consumption")
    #ax2.plot(powerdata.loc[destimes,'Time(Local)'],powerdata.loc[destimes,"Production"],"tab:red", label="National Power production")
    ax2.plot(powerdata.loc[destimes,'Time(Local)'],powerdata.loc[destimes,"Production"]-powerdata.loc[destimes,"Consumption"],"tab:green", label="National Power deficit/surplus")
    #ax1.plot(pdtimes,np.convolve(avg,win*[1/win],"same"),label="50Hz Signal energy",color="tab:blue")
    #ax1.plot(pdtimes,avg,label="50Hz Signal energy",color="tab:blue")
    #ax1.plot(importdata.loc[destimes,'Time(Local)'],df.resample("H").mean(),"tab:blue", label="50Hz Signal")
    #ax2.plot(powerdata.loc[destimes,'Time(Local)'],powerdata.loc[destimes,"Consumption"],"tab:green",label="National Power consumption")
    #ax2.plot(powerdata.loc[destimes,'Time(Local)'],powerdata.loc[destimes,"Production"],"tab:red", label="National Power production")
    #ax2.plot(importdata.loc[destimes,'Time(Local)'],importdata.loc[destimes,"Import"],"tab:red", label="National Power Import")
    ##ax2.plot(importdata.loc[destimes,'Time(Local)'],importdata.loc[destimes,"Export"],"tab:red", label="National Power Export")
    #ax2.plot(importdata.loc[destimes,'Time(Local)'],importdata.loc[destimes,"SumImportExport"],"tab:orange", label="Import + Export")

    ax1.set_title("50Hz Signal energy with power statistics")
    ax1.set_xlabel("Time")
    ax1.set_ylabel("Power Density 1/Hz]")
    #ax1.tick_params(axis="y")  
    ax2.set_ylabel("Power [MWh]")
    #ax2.tick_params(axis="y",)
    #ax1.set_xticks(powerdata.loc[destimes,"Time(Local)"])
    ax1.xaxis.set_major_locator(matplotlib.ticker.MaxNLocator(nbins=25))
    fig.legend(loc="upper left")  
    fig.autofmt_xdate()  
    #fig.tight_layout()
    fig.set_size_inches(16,10)
    fig.savefig(f"50hz vs import_year={starttime[:4]}_sum={sumvar}.png")
    fig.savefig(f"50hz vs import_year={starttime[:4]}_sum={sumvar}.pdf")
    
    plt.show(block=True)






def plot_sidelobes2(data, ndays, lower_freq=50.1, upper_freq=70,starttime="2022-12-01T00:00",timerange=("2023-03-13T04:00","2023-03-13T20:00"),normalized= True):
    fft = rfft(data,axis=1)
    freqs = rfftfreq(data.shape[1],1/210)
    index_array_01hz = (49.9 < freqs) & (freqs < 50.1) 

    upper_sidelobe_index_array = ((lower_freq < freqs ) & (freqs<upper_freq))
    

    times = np.arange(np.datetime64(starttime), np.datetime64(starttime)+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(fft.shape[0]-1),"ms"))
    time_index = (np.datetime64(timerange[0]) <times) &  (times<np.datetime64(timerange[1]))
    print(time_index.shape)
    print(index_array_01hz.shape)
    
    avg_50hz = np.average(np.abs(fft[:, index_array_01hz]), axis=1)

    highlvls = np.linspace(np.min(np.log(np.abs(fft[:,upper_sidelobe_index_array]).T/avg_50hz)),np.max(np.log(np.abs(fft[:,upper_sidelobe_index_array]).T/avg_50hz)),1000)
    log_upp_lobe= np.log(np.abs(fft[:,upper_sidelobe_index_array]).T/avg_50hz)
    
    
    nn_log_upp_lobe=np.log(np.abs(fft[:,upper_sidelobe_index_array])).T
    lobe=fft[:,upper_sidelobe_index_array]
    normalized_lobe=np.divide(lobe.T,avg_50hz).T
    
    timed_lobe= np.log(np.abs(lobe[time_index])).T
    normalized_timed_lobe=np.log(np.abs(normalized_lobe[time_index])).T
    
    lvls = np.linspace(np.min(timed_lobe), np.max(timed_lobe),1000)

    normalized_lvls = np.linspace(np.min(normalized_timed_lobe),np.max(normalized_timed_lobe),1000)
    print("here")
    print(times[time_index].shape)
    print(freqs[upper_sidelobe_index_array].shape)
    print(timed_lobe.shape)
    
    
    plt.figure(figsize=(16,9))
    plt.contour(times[time_index],
                freqs[upper_sidelobe_index_array],
               timed_lobe,levels=lvls,cmap="jet")
    plt.title("Spectrogram")
    plt.xlabel("Date")
    plt.ylabel("Frequency")
    plt.colorbar()
    plt.show()
    """
    plt.figure(figsize=(16,9))
    #print("-------")
    #print(times[time_index].shape)
    #print(freqs[upper_sidelobe_index_array].shape)
    #print(normalized_timed_lobe.shape)
    #print(normalized_lvls.shape)
    plt.contour(times[time_index],
                freqs[upper_sidelobe_index_array],
               normalized_timed_lobe,levels=normalized_lvls,cmap="jet")
    plt.title("Normalized Spectrogram")
    plt.xlabel("Date")
    plt.ylabel("Frequency")
    plt.colorbar()
    plt.show()
    quit()
    """




def train_detection(searchstring,tstart="2023-03-01T00:20:23",tstop="2023-03-01T00:30:32"):
    print(searchstring,tstart,tstop)
    print(tstart.tz_convert("utc"))
    print(tstart)
    tstart=tstart.tz_convert("utc")
    tstop=tstop.tz_convert("utc")
    
    y1,y2=[],[]
    #print("globres: ", list(pl.Path("RawData/march/01/").glob(f"output-20230301-002*")))
    for file in pl.Path("RawData/march/01/").glob(searchstring):
        #print(file)
        data, _ = sf.read(file)
        y1.append(data[:,0])
        y2.append(data[:,1])

    y1 = np.concatenate(y1)
    y2 = np.concatenate(y2)

    
    decimatefactors=[2,5,7,7]
    fs=44100//np.prod(decimatefactors)
    for i in decimatefactors:
        y1 = sig.decimate(y1, i)
        y2 = sig.decimate(y2, i)

    s1=y2-y1
    #initfilt=sig.butter(10,45,"lowpass",fs=fs,output="sos")
    #s1=sig.sosfiltfilt(initfilt,s1)
    step=int(fs*2)
    winlen=int(fs*4)
    print(fs)
    N_partitions=[np.arange(step*i,step*i+winlen,dtype="int64") for i in range((len(s1)-winlen)//step)]
    #print(len(N_partitions[-1]))
    #print(s1.shape)
    #print(np.factor)
    split=[s1[part] for part in N_partitions]
    #print(len(split[0]), len(split[-1]))
    split=np.row_stack(split)
    #print(split)
    #print(split.shape)
    freqs=rfftfreq(split.shape[1],1/fs)
    spectimes=pd.date_range(start=tstart,end=tstop,periods=len(N_partitions))
    colormap=np.log(np.abs(rfft(split,axis=1))).T
    #print(colormap.shape)
    #print(freqs.shape)
    #print(spectimes.shape)
    
    lvls=np.linspace(np.min(colormap),np.max(colormap),1000)
    
    fig,ax=plt.subplots(figsize=(16,9))
    ax.locator_params(axis="x",nbins=6)
    cs = ax.contourf(spectimes,freqs,colormap,levels=lvls,cmap="jet")
    ax.set_xlabel("Time")
    ax.set_ylabel("Frequency")
    fig.colorbar(cs,ax=ax)
    plt.savefig(f"plots/march/Train_detection/90hz/utc_compensated/start={tstart.hour}-{tstart.minute}_spectrogram.png")

    print("here")
    #plt.show()
    
    #plt.specgram(s1,Fs=fs,cmap="jet")
    
    
    
    freqranges=[(1,35),(1,10),(8,12),(20,40)]
    sos=[sig.butter(5,freqrange,"bandpass",fs=fs,output="sos") for freqrange in freqranges]
    filtsigs=[sig.sosfiltfilt(filt,s1) for filt in sos]
    
    #freqs=rfftfreq(len(s1),1/fs)
    time=pd.date_range(start=tstart,end=tstop,periods=s1.flatten().shape[0])
    fig,axes=plt.subplots(len(freqranges),1,sharex=True,figsize=(16,9))
    for signal,freqrange,ax in zip(filtsigs,freqranges,axes):
        ax.plot(time[20:-21],signal[20:-21],label=f"filtered at {freqrange}Hz")
        ax.legend(loc="upper left")
        ax.set_xlabel("Time")
        ax.set_ylabel("Signal [au]")
        ax.set_ylabel("Signal [au]")
    fig.savefig(f"plots/march/Train_detection/90hz/utc_compensated/start={tstart.hour}-{tstart.minute}_filtering.png")
    #plt.figure()
    #plt.plot(rfftfreq(len(y1),1/fs),np.abs(rfft(y1))**2,label="FFT y1")
    #plt.plot(rfftfreq(len(y2),1/fs),np.abs(rfft(y2))**2,label="FFT y2")
    #plt.plot(rfftfreq(len(s1),1/fs),np.abs(rfft(s1))**2,label="FFT s1")
    #plt.ylabel("PDS")
    #plt.xlabel("Frequency [Hz]")
    #plt.title("Power Spectral density")
    #plt.legend()
    #plt.show(block=True)

    

font= {'weight' : 'normal','size'   : 16}
plt.rc("font",**font)
mediumsize=16
plt.rc("xtick",labelsize=mediumsize-2)
plt.rc("ytick",labelsize=mediumsize-2)
plt.rc("axes",labelsize=mediumsize)



march_data_diff,march_data_sum=load_week("march","0123","0123456789")
plot_sidelobes(march_data_diff,31,65,70,starttime="2023-03-31",normalized=False)
plot_sidelobes(march_data_diff,31,65,70,starttime="2023-03-31",normalized=True)
quit()
#march_data_diff,march_data_sum=load_week("march","0123","0123456789")
#plot50hz_and_power_data(march_data_diff,starttime="2023-03-01T00:00",endtime="2023-03-31 23:00:00",ndays=31,sumvar=False)

#train_detection("output-20230301-002*",tstart="2023-03-01T00:20:23",tstop="2023-03-01T00:30:32")

#train_detection("output-20230301-013*",tstart="2023-03-01T01:20:24",tstop="2023-03-01T01:40:30")
#train_detection("output-20230301-021*",tstart="2023-03-01T02:10:55",tstop="2023-03-01T02:20:02")
#train_detection("output-20230301-072*",tstart="2023-03-01T07:20:48",tstop="2023-03-01T07:30:57")
timearr = pd.read_csv("traintimes.txt",delimiter=",",skiprows=1,dtype="str",names=("key","time"))
#print(timearr)

k=1
for  key, starttime in zip(timearr["key"],timearr["time"]):
    
    k+=1
    train_detection(f"output-20230301-{key}*",tstart=pd.Timestamp(starttime,tz="CET"),tstop=pd.Timestamp(starttime,tz="CET")+pd.Timedelta(10,"min"))
    plt.close()
    #if k % 5==1 and k!=1:
        #plt.show(block=True)
quit()
"""
starttime="2023-03-01T00:00"
endtime="2023-03-31 23:00:00"
#print(pd.date_range(start=starttime,end=endtime,freq="H"))
print(pd.Timestamp(powerdata.loc[0,"Time(Local)"]))
print(pd.Timestamp(powerdata.loc[powerdata.shape[0]-1,"Time(Local)"]))
powerdata["Time(Local)"] = pd.date_range(start=pd.Timestamp(powerdata.loc[0,"Time(Local)"]), end=pd.Timestamp(powerdata.loc[powerdata.shape[0]-2,"Time(Local)"]),freq="H")
print(powerdata.head())
destimes=powerdata.isin(pd.date_range(start=pd.Timestamp(starttime),end=pd.Timestamp(endtime),freq="H")).to_numpy()
#starttime="2022-12-01T00:00"
ndays=31
quit()




march_data_diff,march_data_sum=load_week("march","0123","0123456789")

bandpass_energy(march_data_diff,(50.1,85),ndays=31,starttime="2022-12-01T00:00")

bandpass_energy(march_data_diff,(58,62),ndays=31,starttime="2023-03-01T00:00")

bandpass_energy(march_data_diff,(50.1,51),ndays=31,starttime="2023-03-01T00:00")


quit()
march13diff, march13sum = load_week("march", "0", "1")
print(march13diff.shape)
starttimes="2023-03-01T00:00"
plot_sidelobes2(data=march13diff,ndays=1,lower_freq=0,upper_freq=60,starttime=starttimes,timerange=(np.datetime64(starttimes),np.datetime64(starttimes) + np.timedelta64(1,"D")))
bandpass_energy(march13diff,(58,62),ndays=1)
bandpass_energy(march_data_diff,(58,62),ndays=31,starttime="2023-03-01T00:00")
bandpass_energy(march_data_diff,(50.1,65),ndays=31,starttime="2023-03-01T00:00")
         
#des_data_diff,des_data_sum=load_week("des","0123","0123456789")
#plot_temp_and_ranged_spectrogram(des_data_diff, 31,"2022-12-01T00:00",(65,75),0,False) 


quit()
"""



"""


y1,y2=[],[]

for file in pl.Path("RawData/march/01/").glob(f"**/output-20230301-*"):
    print(file)
    data, _ = sf.read(file)
    y1.append(data[:,0])
    y2.append(data[:,1])

y1 = np.concatenate(y1)
y2 = np.concatenate(y2)
decimatefactors=[2,3,5,6,7]
for i in decimatefactors:
    y1 = sig.decimate(y1, i)
    y2 = sig.decimate(y2, i)

day_diff, day_sum = y1-y2, y1+y2
starttime="2023-03-01T00:00"
pdtimes= pd.date_range(start=starttime,end="2023-04-01T00:00",periods=y1.shape[0])
plt.plot(pdtimes[1:],day_diff[1:])
plt.show()
plt.specgram(day_diff[1:],Fs=210,cmap="jet")
plt.show()
#freqrange=(68,71)
#sos=sig.butter(10,freqrange,"bandpass",output="sos",fs=35)
#plt.plot(pdtimes,sig.sosfilt(sos,day_diff),label="bandpassed original downsampled signal")
#plt.show()


"""
print("here 826")
march_35hz_diff,march_35hz_sum=load_35hz("march","0123","0123456789")
#march_35hz_diff2,march_35hz_sum2=load_35hz("march","0","1")
starttime="2023-03-01"
ndays=31




#times = np.arange(np.datetime64(starttime), np.datetime64(starttime)+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(march_35hz_diff.flatten().shape[0]-1),"ms"))
pdtimes = pd.date_range(start=starttime,end="2023-04-01T00:00",periods=march_35hz_diff.shape[0])



fft=np.log(np.abs(np.fft.rfft(march_35hz_diff,axis=1))).T
freqs=np.fft.rfftfreq(march_35hz_diff.shape[1],1/35)
#pdtimes2 = pd.date_range(start=starttime,end="2023-03-02T00:00",periods=march_35hz_diff2.flatten().shape[0])

plt.figure(figsize=(16,9))
print(pdtimes.shape)
print(freqs.shape)
print(fft.shape)
lvls=np.linspace(np.min(fft),np.max(fft),100)
plt.contour(pdtimes,
            freqs,
            fft,levels=lvls,cmap="jet")
plt.title("Spectrogram")
plt.xlabel("Date")
plt.ylabel("Frequency")
plt.colorbar()
plt.show()




#plt.plot(pdtimes2,march_35hz_diff2.flatten(),label="Signal at 35Hz sampling rate")
#plt.plot(pdtimes,march_35hz_diff.flatten(),label="Signal at 35Hz sampling rate with artifact removal")
#plt.show()

fft2=np.log(np.abs(np.fft.rfft(march_35hz_diff2,axis=1))).T
freqs2=np.fft.rfftfreq(march_35hz_diff2.shape[1],1/35)
pdtimes2 = pd.date_range(start=starttime,end="2023-03-02T00:00",periods=march_35hz_diff2.shape[0])


plt.figure(figsize=(16,9))
print(pdtimes2.shape)
print(freqs.shape)
print(fft.shape)
lvls=np.linspace(np.min(fft),np.max(fft),100)
plt.contour(pdtimes2,
            freqs2,
            fft2,levels=lvls,cmap="jet")
plt.title("Spectrogram")
plt.xlabel("Date")
plt.ylabel("Frequency")
plt.colorbar()
plt.show()




#plt.plot(pdtimes,sig.sosfilt(sos,march_35hz_diff.flatten()),label=f"{freqrange} bandpassed signal")
#plt.legend()

#plt.show()
plt.specgram(march_35hz_diff.flatten(),Fs=35,cmap="jet")
plt.colorbar()
plt.show()
"""
n=35*30
ma_win=n*[1/n]
n2=35*60*5
ma_win2=n2*[1/n2]
fig, ax1=plt.subplots()
ax2 = ax1.twinx()

#ax1.plot(pdtimes,np.convolve(ma_win,march_35hz_diff.flatten(),"same"))
#ax1.plot(pdtimes,np.convolve(ma_win,march_35hz_diff.flatten(),"same"))

#plt.show()


df=pd.Series(march_35hz_diff.flatten(),index=pdtimes)
pdhours=pd.date_range(start=pd.Timestamp(starttime),end=pd.Timestamp("2023-04-01"),freq="H")

weatherdata=pd.read_csv("weatherdata_march.csv",sep=";",skipfooter=1)
print(weatherdata.columns)
print(weatherdata["Lufttemperatur"].shape)
print(pdhours[:-1].shape)

ax1.plot(pdtimes,np.convolve(ma_win,march_35hz_diff.flatten(),"same"))
ax1.plot(pdtimes,np.convolve(ma_win2,march_35hz_diff.flatten(),"same"))
ax2.plot(pdhours[:744],weatherdata["Lufttemperatur"])
plt.show()


plt.plot(pdtimes,df)

plt.show()

quit()
"""


#plot50hz_and_power_data(march_data_sum,starttime="2023-03-01T00:00",endtime="2023-03-31 23:00:00",ndays=31,sumvar=True)
#des_data_diff,des_data_sum=load_week("des","0123","0123456789")

#spec_only_start_of_segments(march_data_diff,1)

#plot50hz_and_power_data(des_data_diff,starttime="2022-12-01T00:00",endtime="2022-12-31 23:00:00",ndays=31,sumvar=False)
#plot50hz_and_power_data(des_data_sum,starttime="2022-12-01T00:00",endtime="2022-12-31 23:00:00",ndays=31,sumvar=True)

#plot_sidelobes(march_data_diff, 31,starttime="2023-03-01T00:00")
#plot_temp_and_ranged_spectrogram(march_data_diff,31,"2023-03-01T00:00",(50.1,58),0,False)
print("here1")
#plot_temp_and_ranged_spectrogram(march_data_diff, 31,"2023-03-01T00:00",(50.1,70),0,True)
print("here2")
#plot_temp_and_ranged_spectrogram(march_data_diff, 31,"2023-03-01T00:00",(30,49.9),0,False)
print("here3")
#plot_temp_and_ranged_spectrogram(march_data_diff, 31,"2023-03-01T00:00",(30,49.9),0,True)
print("here4")

#des_data_diff,des_data_sum=load_week("des","0123","0123456789")
#plot_sidelobes(des_data_diff,31)
#plot_temp_and_ranged_spectrogram(des_data_diff, 31,"2022-12-01T00:00",(50.1,70),0,True)
#print("here5")
#plot_temp_and_ranged_spectrogram(des_data_diff, 31,"2022-12-01T00:00",(50.1,70),0,False) 
#print("here6")
#plot_temp_and_ranged_spectrogram(des_data_diff, 31,"2022-12-01T00:00",(30,49.9),0,False)
#print("here7")
#plot_temp_and_ranged_spectrogram(des_data_diff, 31,"2022-12-01T00:00",(30,49.9),0,True)
#print("here8")
#quit()
#peak_track_corr_temp(march_data_diff,"weatherdata_march.csv",(49.9,50.1))


#bandpass_energy(march_data_diff,(0,8),ndays=31)

#bandpass_energy(march_data_diff,(8,12),ndays=31)
#bandpass_energy(march_data_diff,(12,35),ndays=31)
#bandpass_energy(march_data_diff,(0,35),ndays=31)

#plot_temp_and_ranged_spectrogram(march_data_diff,31,starttime="2023-03-01T00:00",freq_range=(69,71))


#plot_sidelobes(load_week("march","0123","123456789")[0],31,starttime="2023-03-01T00:00")
#plot_temp_and_ranged_spectrogram(load_week("march","0123","123456789")[0],31,starttime="2023-03-01T00:00",freq_range=(67,71))
#plot_temp_and_ranged_spectrogram(load_week("march","0123","123456789")[0],31,starttime="2023-03-01T00:00",freq_range=(49.9,50.1))
#plot_sidelobes(load_week("march","0123","123456789")[0],31,starttime="2023-03-01T00:00",freq_range=(70,105))
#plot_temp_and_ranged_spectrogram(march_data_diff,31,starttime="2023-03-01T00:00",freq_range=(8,20))

#plot_temp_and_ranged_spectrogram(load_week("march","0123","123456789")[0],31,starttime="2023-03-01T00:00",freq_range=(70,105))
#plot_temp_and_ranged_spectrogram(load_week("march","0123","123456789")[0],31,starttime="2023-03-01T00:00",freq_range=(10,20))
#plot_temp_and_ranged_spectrogram(load_week("des","0123","123456789")[0],31,starttime="2022-12-01T00:00",freq_range=(65,75))
#plot_temp_and_ranged_spectrogram(load_week("des","0123","123456789")[0],31,starttime="2022-12-01T00:00",freq_range=(25,35))

quit()
#sop_diff, sop_sum = load_week("0","1-7")
#print(sop_diff.shape)
#diff_fft = rfft(sop_diff, axis=1)
#freqs = rfftfreq(sop_diff.shape[1], 1/210)

"""


#index_array_01hz = freqs > 49.95




plt.plot(freqs[low:high],np.abs(diff_fft[0,low:high]))

plt.show()
plt.plot(np.linspace(1,7,sop_diff.shape[0]),np.sum(np.abs(diff_fft[:, index_array_rest]),axis=1), label="Full signal")
plt.plot(np.linspace(1,7,sop_diff.shape[0]),np.sum(np.abs(diff_fft[:, index_array_2hz]),axis=1),label="2Hz range")
plt.plot(np.linspace(1,7,sop_diff.shape[0]),np.sum(np.abs(diff_fft[:, index_array_01hz]),axis=1), label="0.1Hz range")

plt.legend()
plt.show()





#plot_contour(sop_diff,7)






######            frequency bin plots. ###########################

plt.plot(times,np.average(np.abs(month_fft[:, index_array_rest]),axis=1), label=f"(0, {lower_carried_freq})|({upper_carried_freq}, 105)Hz range")
plt.plot(times,np.average(np.abs(month_fft[:, index_array_2hz_lower]),axis=1), label=f"({lower_carried_freq}, {lower_normal_freq})Hz range")
plt.plot(times,np.average(np.abs(month_fft[:, index_array_2hz_upper]),axis=1), label=f"({upper_normal_freq}, {upper_carried_freq})Hz range")

plt.plot(times,np.average(np.abs(month_fft[:, index_array_01hz]),axis=1), label=f"({lower_normal_freq}, {upper_normal_freq}) range")
plt.title("Average PDS with respect to 50Hz carrier wave, differential signal")
plt.legend()
plt.show()

"""


"""
ndays=31    
plt.plot(times,np.average(np.abs(sum_fft[:, index_array_rest]),axis=1), label="(0, 49)|(51, 105)Hz range")
plt.plot(times,np.average(np.abs(sum_fft[:, index_array_2hz]),axis=1),label="(49, 49.95)|(50.05, 51)Hz range")
plt.plot(times,np.average(np.abs(sum_fft[:, index_array_01hz]),axis=1), label="(49.95, 50.05) range")
plt.title("Average PDS with respect to 50Hz carrier wave, sum signal")

plt.legend()
plt.show()
"""

"""
plt.plot(times,np.average(np.abs(month_fft[:, index_array_2hz]),axis=1), label="(49, 49.95)|(50.05, 51)Hz range, differential signal")
plt.plot(times,np.average(np.abs(sum_fft[:, index_array_2hz]),axis=1),label="(49, 49.95)|(50.05, 51)Hz range, sum signal")

plt.legend()
plt.show()


plt.plot(times,np.average(np.abs(month_fft[:, index_array_01hz]),axis=1), label="(49.95, 50.05) range, differential signal")
plt.plot(times,np.average(np.abs(sum_fft[:, index_array_01hz]),axis=1), label="(49.95, 50.05) range, sum signal")


month_diff,month_sum=load_week("des","0123", "0123456789")
month_fft = rfft(month_diff, axis=1)
sum_fft = rfft(month_sum,axis=1)
freqs = rfftfreq(month_diff.shape[1], 1/210)

low,high=2980,3020 


lower_normal_freq, upper_normal_freq = 49.9, 50.1
lower_carried_freq,upper_carried_freq = 30, 70


index_array_01hz = (lower_normal_freq < freqs) & (freqs < upper_normal_freq) 
index_array_2hz_lower = ((lower_carried_freq < freqs) & (freqs < lower_normal_freq)) 
index_array_2hz_upper = ((upper_normal_freq < freqs ) & (freqs<upper_carried_freq))
index_array_rest =  (freqs<lower_carried_freq) | (freqs>upper_carried_freq)

ndays=31
times=np.arange(np.datetime64("2022-12-01T00:00"), np.datetime64("2022-12-01T00:00")+np.timedelta64(ndays,"D"), np.timedelta64((ndays)*86400*1000//(month_fft.shape[0]-1),"ms"))
avg_50hz = np.average(np.abs(month_diff[:, index_array_01hz]), axis=1)

# prod_comp_national=pd.read_excel("ProductionConsumption-2022.xls")





plt.plot(times, avg_50hz, label="50hz signal")
plt.plot(times[peaks], avg_50hz[peaks],"x",label="peaks")
plt.plot(times, moving_avg,label="moving average signal")
plt.plot(times, extraweather(np.linspace(1,31,len(times))) / max(weatherdata["Lufttemperatur"]) * max(avg_50hz))

plt.legend()
plt.show()





low_lvls = np.linspace(np.min(np.log(np.abs(month_fft[:,index_array_2hz_lower]).T/avg_50hz)),np.max(np.log(np.abs(month_fft[:,index_array_2hz_lower]).T/avg_50hz)),100)
high_lvls = np.linspace(np.min(np.log(np.abs(month_fft[:,index_array_2hz_upper]).T/avg_50hz)),np.max(np.log(np.abs(month_fft[:,index_array_2hz_upper]).T/avg_50hz)),100)
plt.contour(times,
             freqs[index_array_2hz_lower],
             np.log(np.abs(month_fft[:,index_array_2hz_lower]).T/avg_50hz),levels=low_lvls,cmap="jet")
plt.title("Normalised spectrogram of the lower sidelobe")
plt.xlabel("date")
plt.ylabel("frequency")
plt.colorbar()
plt.savefig("upper sidelobe normalized on carrierwave.png")
plt.savefig("upper sidelobe normalized on carrierwave.pdf")
plt.show()

plt.contour(times,
             freqs[index_array_2hz_upper],
             np.log(np.abs(month_fft[:,index_array_2hz_upper]).T/avg_50hz),levels=high_lvls,cmap="jet")
plt.title("Normalised spectrogram of the upper sidelobe")
plt.xlabel("date")
plt.ylabel("frequency")
plt.colorbar()

plt.show()

weatherdata=pd.read_csv("weather_sarpsborg.csv",sep=";",skipfooter=1)
extraweather=sci.interpolate.interp1d(np.linspace(1,31,len(weatherdata["Lufttemperatur"])),weatherdata["Lufttemperatur"])

# prod_comp_national=pd.read_excel("ProductionConsumption-2022.xls")




peaks, _ = sig.find_peaks(avg_50hz,distance=500,prominence=100)
plt.plot(times, avg_50hz, label="50hz signal")
plt.plot(times[peaks], avg_50hz[peaks],"x",label="peaks")
plt.plot(times, moving_avg,label="moving average signal")
plt.plot(times, extraweather(np.linspace(1,31,len(times))) / max(weatherdata["Lufttemperatur"]) * max(avg_50hz))

plt.legend()
plt.show()





extrapolated = sci.interpolate.interp1d(np.linspace(1,31,len(peaks)),avg_50hz[peaks])
extra2 = extrapolated(np.linspace(1,31,len(weatherdata["Lufttemperatur"])))

plt.plot(np.linspace(1,31,len(weatherdata["Lufttemperatur"])),weatherdata["Lufttemperatur"])
plt.plot(np.linspace(1,31,len(weatherdata["Lufttemperatur"])),extra2)
plt.show()

corr=np.correlate(extra2,weatherdata["Lufttemperatur"])
cov50hz = np.cov(extra2)
covtemp = np.cov(weatherdata["Lufttemperatur"])
print(cov50hz)
print(covtemp)

print(corr/(cov50hz*covtemp))

extrapolated_month_weather=extraweather(np.linspace(1,31,len(times)))
fullcorr=np.correlate(extrapolated_month_weather/np.cov(extrapolated_month_weather),avg_50hz/np.cov(avg_50hz),mode="full")

plt.plot(fullcorr)
print(max(fullcorr))
plt.title("Crosscorrelation, temperature and 50hz signal. ")
plt.show()

plt.plot(avg_50hz,extraweather(np.linspace(1,31,len(times))))
plt.show()
quit()



plt.plot(times,diff_2hz_avg/avg_50hz,label="normalized carried wave")
plt.legend()
plt.show()
plt.plot(times,diff_2hz_avg/avg_50hz,label="normalized carried wave")
plt.plot(times,np.average(np.abs(sum_fft[:, index_array_2hz])/avg_50hz,axis=1),label="(49, 49.95)|(50.05, 51)Hz range")

plt.legend()
plt.show()
quit()
plot_contour(month_diff,31)
"""